local sdk = require "sdk.sdk"
local config = require "config"
local drawManager = require "drawManager"
local player = require "player"
local Orc = require "entities.Orc"
local OtherPlayer = require "entities.OtherPlayer"
local Spawner = require "entities.Spawner"

local entityMapping = {orc = Orc, player = OtherPlayer}
local entityManager = {entities = {}, pendingRemoval = {}}

function entityManager.load()
    entityManager.entities = {}
    entityManager.pendingRemoval = {}
end

function checkCollision(x1, y1, w1, h1, x2, y2, w2, h2)
    if x1 == nil or x2 == nil then return false end
    return x1 < x2 + w2 and x2 < x1 + w1 and y1 < y2 + h2 and y2 < y1 + h1
end

function entityManager.update(dt)
    local seenKeys = {}
    sdk.update()

    for k, v in pairs(sdk.entities) do
        if k == sdk.uuid then
            player:setDisplayName(v.Data.displayName)
            player:setScore(v.Data.score)
        elseif v.Type == 'spawner' then
            Spawner:update(v.Data)
        else
            if entityMapping[v.Type] then
                if not entityManager.entities[k] then
                    local newEntity = entityMapping[v.Type].new(k, v.X /
                                                                    config.worldScale,
                                                                v.Y /
                                                                    config.worldScale,
                                                                v.Data)
                    newEntity:load()
                    entityManager.entities[k] = newEntity
                end
                entityManager.entities[k]:update(dt, v.X / config.worldScale,
                                                 v.Y / config.worldScale, v.Data)
                seenKeys[k] = true
            end
        end
    end

    if player.state == 'attack' then
        for k, entity in pairs(entityManager.entities) do
            if entity.getBoundingBox then
                local x1, y1, w1, h1 = player:getBoundingBox()
                local x2, y2, w2, h2 = entity:getBoundingBox()

                if checkCollision(x1, y1, w1, h1, x2, y2, w2, h2) then
                    entity:onCollision()
                end
            end
        end
    end

    for k, v in pairs(sdk.entities) do
        if v.Type ~= "player" and entityMapping[v.Type] then
            local entity = entityManager.entities[k]
            if entity.getBoundingBox and entity.state == 'attack' then
                local x1, y1, w1, h1 = entity:getBoundingBox()
                local x2, y2, w2, h2 = player:getBoundingHurtBox()

                if checkCollision(x1, y1, w1, h1, x2, y2, w2, h2) then
                    player:onCollision(10, entity)
                end
            end
        end
    end

    for k, record in pairs(entityManager.pendingRemoval) do
        record.timer = record.timer - dt
        if record.timer <= 0 then entityManager.pendingRemoval[k] = nil end
        record.entity:update(dt, record.entity.x, record.entity.y,
                             {state = record.entity.state})
    end

    for k, entity in pairs(entityManager.entities) do
        if not seenKeys[k] then
            entityManager.pendingRemoval[k] =
                {entity = entity, timer = entity.state == 'die' and 1 or 0}
            entityManager.entities[k] = nil
        end
    end
end

function entityManager.draw()
    for k, v in pairs(entityManager.entities) do v:draw() end

    for k, record in pairs(entityManager.pendingRemoval) do
        record.entity:draw()
    end
end

return entityManager
