local utf8 = require "utf8"
local sdk = require "sdk.sdk"
local config = require "config"
local caretTime, caretVisible = 0, false
local pixelFont = love.graphics.newFont("font/PixelAE-Regular.ttf", 16)
local defaultFont = love.graphics.newFont()
local loginMenu = {
    username = "",
    password = "",
    authStatus = "Please log in",
    inputs = {
        username = {
            x = 100,
            y = 100,
            width = 200,
            height = 30,
            isFocused = true,
            text = "",
            label = "Username:"
        },
        password = {
            x = 100,
            y = 160,
            width = 200,
            height = 30,
            isFocused = false,
            text = "",
            label = "Password:",
            mask = true
        }
    },
    button = {x = 100, y = 210, width = 200, height = 40, text = "Log in"}
}

local function drawInputBox(input)
    love.graphics.setColor(1, 1, 1)
    love.graphics.print(input.label, input.x, input.y - 20)
    love.graphics.rectangle("line", input.x, input.y, input.width, input.height)
    if input.mask then love.graphics.setFont(defaultFont) end
    local displayText = input.mask and string.rep("•", #input.text) or
                            input.text
    local textWidth = love.graphics.getFont():getWidth(displayText)
    if textWidth > input.width - 10 then
        displayText = displayText:sub(-math.floor(
                                          (input.width - 10) /
                                              (textWidth / #displayText)))
    end
    love.graphics.print(displayText, input.x + 5, input.y + 5)
    love.graphics.setFont(pixelFont)
end

local function updateCaret(dt)
    caretTime = caretTime + dt
    if caretTime >= 0.5 then
        caretVisible = not caretVisible
        caretTime = 0
    end
end

local function drawCaret(input)
    if input.isFocused and caretVisible then
        if input.mask then love.graphics.setFont(defaultFont) end
        local textWidth = love.graphics.getFont():getWidth(input.text)
        local caretX = input.x + 5 +
                           (input.mask and
                               love.graphics.getFont():getWidth(
                                   string.rep("•", #input.text)) or textWidth)
        caretX = math.min(caretX, input.x + input.width - 5)
        love.graphics.line(caretX, input.y + 5, caretX, input.y + 25)
        love.graphics.setFont(pixelFont)
    end
end

function loginMenu.load()
    if love.system.getOS() == "Android" or love.system.getOS() == "iOS" then
        love.keyboard.setTextInput(true, loginMenu.inputs.username.x,
                                   loginMenu.inputs.username.y,
                                   loginMenu.inputs.username.width,
                                   loginMenu.inputs.username.height)
    else
        love.keyboard.setTextInput(true)
    end
    love.mouse.setVisible(true)
    love.keyboard.setKeyRepeat(true)
    loginMenu.inputs.username.isFocused = true
    loginMenu.inputs.password.isFocused = false
    loginMenu.authStatus = "Please log in"
    for _, input in pairs(loginMenu.inputs) do input.text = "" end
    love.graphics.setFont(pixelFont)
end

function loginMenu.textinput(t)
    for _, input in pairs(loginMenu.inputs) do
        if input.isFocused then input.text = input.text .. t end
    end
end

local function login()
    loginMenu.button.text = "Logging in"
    sdk.init(config.gameId, loginMenu.inputs.username.text,
             loginMenu.inputs.password.text)
    loginMenu.authStatus = sdk.uuid and "Login successful" or "Login failed"
    loginMenu.button.text = "Log in"
    if sdk.uuid then
        love.keyboard.setTextInput(false)
        loginMenu.inputs.username.text = ""
        loginMenu.inputs.password.text = ""
    end
end

function loginMenu.mousepressed(x, y, button, istouch, presses)
    if button == 1 then
        for _, input in pairs(loginMenu.inputs) do
            if x > input.x and x < input.x + input.width and y > input.y and y <
                input.y + input.height then
                input.isFocused = true
                if love.system.getOS() == "Android" or love.system.getOS() ==
                    "iOS" then
                    love.keyboard.setTextInput(true, input.x, input.y,
                                               input.width, input.height)
                end
            else
                input.isFocused = false
            end
        end
        if x > loginMenu.button.x and x < loginMenu.button.x +
            loginMenu.button.width and y > loginMenu.button.y and y <
            loginMenu.button.y + loginMenu.button.height then login() end
    end
end

function loginMenu.keypressed(key)
    if (key == "v" and love.keyboard.isDown("lctrl")) or
        (key == "v" and love.keyboard.isDown("rctrl")) or
        (key == "v" and love.keyboard.isDown("lgui")) or
        (key == "v" and love.keyboard.isDown("rgui")) then
        local clipboardText = love.system.getClipboardText()
        local newlinePos = clipboardText:find("\n")
        if newlinePos then
            clipboardText = clipboardText:sub(1, newlinePos - 1)
        end
        for _, input in pairs(loginMenu.inputs) do
            if input.isFocused then
                input.text = input.text .. clipboardText
                break
            end
        end
    elseif key == "tab" then
        if loginMenu.inputs.username.isFocused then
            loginMenu.inputs.username.isFocused = false
            loginMenu.inputs.password.isFocused = true
        elseif loginMenu.inputs.password.isFocused then
            loginMenu.inputs.password.isFocused = false
            loginMenu.inputs.username.isFocused = true
        else
            loginMenu.inputs.username.isFocused = true
        end
        for _, input in pairs(loginMenu.inputs) do
            if input.isFocused == true then
                love.keyboard.setTextInput(true, input.x, input.y, input.width,
                                           input.height)
            end
        end
    elseif key == "return" then
        if love.system.getOS() == "Android" or love.system.getOS() == "iOS" then
            if loginMenu.inputs.username.isFocused then
                loginMenu.inputs.username.isFocused = false
                loginMenu.inputs.password.isFocused = true
                love.keyboard.setTextInput(true, loginMenu.inputs.password.x,
                                           loginMenu.inputs.password.y,
                                           loginMenu.inputs.password.width,
                                           loginMenu.inputs.password.height)
            elseif loginMenu.inputs.password.isFocused then
                login()
            end
        else
            login()
        end
    else
        if key == "backspace" then
            for _, input in pairs(loginMenu.inputs) do
                if input.isFocused then
                    local byteoffset = utf8.offset(input.text, -1)

                    if byteoffset then
                        input.text = string.sub(input.text, 1, byteoffset - 1)
                    end
                end
            end
        end
    end
end

function loginMenu.update(dt) updateCaret(dt) end

function loginMenu.draw()
    love.graphics.setColor(1, 1, 1)
    love.graphics.print(loginMenu.authStatus, 100,
                        loginMenu.inputs.username.y - 50)
    for _, input in pairs(loginMenu.inputs) do
        drawInputBox(input)
        drawCaret(input)
    end
    love.graphics.setColor(0.5, 0.5, 0.5)
    love.graphics.rectangle("fill", loginMenu.button.x, loginMenu.button.y,
                            loginMenu.button.width, loginMenu.button.height)
    love.graphics.setColor(1, 1, 1)
    love.graphics.print(loginMenu.button.text,
                        loginMenu.button.x + (loginMenu.button.width / 2 -
                            love.graphics.getFont():getWidth(
                                loginMenu.button.text) / 2),
                        loginMenu.button.y + 10)
end

return loginMenu
