love.graphics.setDefaultFilter("nearest", "nearest")

local sdk = require "sdk.sdk"
local hud = require "hud"
local menu = require "menu"
local world = require "world"
local player = require "player"

local gameState = "login"
local screenWidth, screenHeight = 0, 0

function love.load()
    screenWidth, screenHeight = love.window.getDesktopDimensions(1)
    if screenWidth and screenHeight then
        love.window.setMode(screenWidth, screenHeight, {fullscreen = true})
    else
        love.window.setMode(800, 600, {fullscreen = false})
    end
    if gameState == "login" then menu.load() end
end

function love.update(dt)
    if not sdk.uuid and gameState ~= "login" then
        gameState = "login"
        menu.load()
    end
    if gameState == "login" then
        menu.update(dt)
        if sdk.uuid then
            gameState = "game"
            world.load(screenWidth, screenHeight)
            hud.load()
        end
    elseif gameState == "game" then
        world.update(dt)
    end
end

function love.draw()
    if gameState == "login" then
        menu.draw()
    elseif gameState == "game" then
        world.draw()
        hud.draw()
    end
end

function love.textinput(t) if gameState == "login" then menu.textinput(t) end end

function love.mousepressed(x, y, button, istouch, presses)
    if gameState == "login" then
        menu.mousepressed(x, y, button, istouch, presses)
    elseif gameState == "game" then
        player:mousepressed(x, y, button)
        hud.mousepressed(x, y, button)
    end
end

function love.keypressed(key)
    if gameState == "login" then
        menu.keypressed(key)
    elseif gameState == "game" then
        player:keypressed(key)
    end
end

function love.touchpressed(id, x, y, dx, dy, pressure)
    if gameState == "game" then
        player:touchpressed(id, x, y, dx, dy, pressure)
    end
end

function love.touchmoved(id, x, y, dx, dy, pressure)
    if gameState == "game" then player:touchmoved(id, x, y, dx, dy, pressure) end
end

function love.touchreleased(id, x, y, dx, dy, pressure)
    if gameState == "game" then
        player:touchreleased(id, x, y, dx, dy, pressure)
    end
end

function love.resize(w, h) screenWidth, screenHeight = w, h end
