local utf8 = require "utf8"
local sdk = require "sdk.sdk"
local config = require "config"
local caretTime, caretVisible = 0, false
local pixelFont = love.graphics.newFont("font/PixelAE-Regular.ttf", 16)
local defaultFont = love.graphics.newFont()
local menu = {
    displayName = "",
    authStatus = "Please set a display name",
    inputs = {
        displayName = {
            x = 100,
            y = 100,
            width = 200,
            height = 30,
            isFocused = true,
            text = "",
            label = "Display Name:"
        }
    },
    button = {x = 100, y = 150, width = 200, height = 40, text = "Join"}
}

local function drawInputBox(input)
    love.graphics.setColor(1, 1, 1)
    love.graphics.print(input.label, input.x, input.y - 20)
    love.graphics.rectangle("line", input.x, input.y, input.width, input.height)
    if input.mask then love.graphics.setFont(defaultFont) end
    local displayText = input.mask and string.rep("•", #input.text) or
                            input.text
    local textWidth = love.graphics.getFont():getWidth(displayText)
    if textWidth > input.width - 10 then
        displayText = displayText:sub(-math.floor(
                                          (input.width - 10) /
                                              (textWidth / #displayText)))
    end
    love.graphics.print(displayText, input.x + 5, input.y + 5)
    love.graphics.setFont(pixelFont)
end

local function updateCaret(dt)
    caretTime = caretTime + dt
    if caretTime >= 0.5 then
        caretVisible = not caretVisible
        caretTime = 0
    end
end

local function drawCaret(input)
    if input.isFocused and caretVisible then
        if input.mask then love.graphics.setFont(defaultFont) end
        local textWidth = love.graphics.getFont():getWidth(input.text)
        local caretX = input.x + 5 +
                           (input.mask and
                               love.graphics.getFont():getWidth(
                                   string.rep("•", #input.text)) or textWidth)
        caretX = math.min(caretX, input.x + input.width - 5)
        love.graphics.line(caretX, input.y + 5, caretX, input.y + 25)
        love.graphics.setFont(pixelFont)
    end
end

function menu.load()
    menu.inputs.displayName.text = ""
    if love.system.getOS() == "Android" or love.system.getOS() == "iOS" then
        love.keyboard.setTextInput(true, menu.inputs.displayName.x,
                                   menu.inputs.displayName.y,
                                   menu.inputs.displayName.width,
                                   menu.inputs.displayName.height)
    else
        love.keyboard.setTextInput(true)
    end
    love.mouse.setVisible(true)
    love.keyboard.setKeyRepeat(true)
    menu.inputs.displayName.isFocused = true
    menu.authStatus = "Please set a display name"
    for _, input in pairs(menu.inputs) do input.text = "" end
    love.graphics.setFont(pixelFont)
end

function menu.textinput(t)
    for _, input in pairs(menu.inputs) do
        if input.isFocused then input.text = input.text .. t end
    end
end

local function login()
    if menu.inputs.displayName.text == "" then
        menu.authStatus = "Display name is required"
        return
    end
    sdk.init(config.gameId, "", "")
    menu.authStatus = sdk.uuid and "Join successful" or "Join failed"
    if sdk.uuid then
        love.timer.sleep(0.5)
        sdk.join()
        sdk.message({displayName = menu.inputs.displayName.text})
        love.keyboard.setTextInput(false)
    end
end

function menu.mousepressed(x, y, button, istouch, presses)
    if button == 1 then
        for _, input in pairs(menu.inputs) do
            if x > input.x and x < input.x + input.width and y > input.y and y <
                input.y + input.height then
                input.isFocused = true
                if love.system.getOS() == "Android" or love.system.getOS() ==
                    "iOS" then
                    love.keyboard.setTextInput(true, input.x, input.y,
                                               input.width, input.height)
                end
            else
                input.isFocused = false
            end
        end
        if x > menu.button.x and x < menu.button.x + menu.button.width and y >
            menu.button.y and y < menu.button.y + menu.button.height then
            login()
        end
    end
end

function menu.keypressed(key)
    if (key == "v" and love.keyboard.isDown("lctrl")) or
        (key == "v" and love.keyboard.isDown("rctrl")) or
        (key == "v" and love.keyboard.isDown("lgui")) or
        (key == "v" and love.keyboard.isDown("rgui")) then
        local clipboardText = love.system.getClipboardText()
        local newlinePos = clipboardText:find("\n")
        if newlinePos then
            clipboardText = clipboardText:sub(1, newlinePos - 1)
        end
        for _, input in pairs(menu.inputs) do
            if input.isFocused then
                input.text = input.text .. clipboardText
                break
            end
        end
    elseif key == "tab" then
        return
    elseif key == "return" then
        login()
    else
        if key == "backspace" then
            for _, input in pairs(menu.inputs) do
                if input.isFocused then
                    local byteoffset = utf8.offset(input.text, -1)

                    if byteoffset then
                        input.text = string.sub(input.text, 1, byteoffset - 1)
                    end
                end
            end
        end
    end
end

function menu.update(dt) updateCaret(dt) end

function menu.draw()
    love.graphics.setColor(1, 1, 1)
    love.graphics.print(menu.authStatus, 100, menu.inputs.displayName.y - 50)
    for _, input in pairs(menu.inputs) do
        drawInputBox(input)
        drawCaret(input)
    end
    love.graphics.setColor(0.5, 0.5, 0.5)
    love.graphics.rectangle("fill", menu.button.x, menu.button.y,
                            menu.button.width, menu.button.height)
    love.graphics.setColor(1, 1, 1)
    love.graphics.print(menu.button.text, menu.button.x +
                            (menu.button.width / 2 -
                                love.graphics.getFont():getWidth(
                                    menu.button.text) / 2), menu.button.y + 10)
end

return menu
