local entityManager = require "entityManager"
local player = require "player"
local miniMap = {dotSize = 2, size = 160, margin = 20, range = 3600}

local colorMapping = {orc = {1, 0, 0}, player = {0, 1, 0}}

function miniMap.draw()
    local screenWidth, screenHeight = love.graphics.getDimensions()
    local minimapX = screenWidth - miniMap.size - miniMap.margin
    local minimapY = miniMap.margin
    local playerX, playerY = player.x, player.y

    love.graphics.setColor(0, 0, 0, 0.5)
    love.graphics.rectangle('fill', minimapX, minimapY, miniMap.size,
                            miniMap.size)

    for _, entity in pairs(entityManager.entities) do
        if entity.type and colorMapping[entity.type] then
            local entityX, entityY = entity.x, entity.y
            local relativeX = (entityX - playerX) / miniMap.range *
                                  (miniMap.size / 2)
            local relativeY = (entityY - playerY) / miniMap.range *
                                  (miniMap.size / 2)

            if math.abs(relativeX) <= miniMap.size / 2 and math.abs(relativeY) <=
                miniMap.size / 2 then
                local r, g, b = colorMapping[entity.type]
                love.graphics.setColor(r, g, b)
                love.graphics.rectangle('fill', minimapX + miniMap.size / 2 +
                                            relativeX,
                                        minimapY + miniMap.size / 2 + relativeY,
                                        miniMap.dotSize, miniMap.dotSize)

            end
        end
    end

    love.graphics.setColor(1, 1, 1)
    love.graphics.rectangle('fill', minimapX + miniMap.size / 2,
                            minimapY + miniMap.size / 2, miniMap.dotSize,
                            miniMap.dotSize)
end

return miniMap
