local sdk = require "sdk.sdk"
local camera = require "camera"
local drawManager = require "drawManager"
local entityManager = require "entityManager"
local player = require "player"
local scale = 0.01
local seed = 4038827759

local viewportWidth = 0
local viewportHeight = 0
local buffer = 5
local tileSize = 32
local tileScale = 4

local world = {}
local map = {}

local tilesetImage = nil
local tileQuads = {}
local currentWaterFrame = 1
local waterFrameTimer = 0
local waterFrameDuration = 0.2
local propsImage = nil
local propsQuads = {}
local propsSeed = 9324234

function world.load(screenWidth, screenHeight)
    love.mouse.setVisible(false)
    love.keyboard.setKeyRepeat(false)
    viewportWidth = screenWidth
    viewportHeight = screenHeight
    player:load()
    tilesetImage = love.graphics.newImage("img/World/Tiles.png")
    tileQuads = {
        water = {
            topLeft = {
                love.graphics.newQuad(200, 24, 8, 8,
                                      tilesetImage:getDimensions()),
                love.graphics.newQuad(232, 24, 8, 8,
                                      tilesetImage:getDimensions())
            },
            left = {
                love.graphics.newQuad(200, 32, 8, 8,
                                      tilesetImage:getDimensions()),
                love.graphics.newQuad(232, 32, 8, 8,
                                      tilesetImage:getDimensions())
            },
            bottomLeft = {
                love.graphics.newQuad(200, 40, 8, 8,
                                      tilesetImage:getDimensions()),
                love.graphics.newQuad(232, 40, 8, 8,
                                      tilesetImage:getDimensions())
            },
            top = {
                love.graphics.newQuad(208, 24, 8, 8,
                                      tilesetImage:getDimensions()),
                love.graphics.newQuad(240, 24, 8, 8,
                                      tilesetImage:getDimensions())
            },
            middle = {
                love.graphics.newQuad(208, 32, 8, 8,
                                      tilesetImage:getDimensions()),
                love.graphics.newQuad(240, 32, 8, 8,
                                      tilesetImage:getDimensions())
            },
            bottom = {
                love.graphics.newQuad(208, 40, 8, 8,
                                      tilesetImage:getDimensions()),
                love.graphics.newQuad(240, 40, 8, 8,
                                      tilesetImage:getDimensions())
            },
            topRight = {
                love.graphics.newQuad(216, 24, 8, 8,
                                      tilesetImage:getDimensions()),
                love.graphics.newQuad(248, 24, 8, 8,
                                      tilesetImage:getDimensions())
            },
            right = {
                love.graphics.newQuad(216, 32, 8, 8,
                                      tilesetImage:getDimensions()),
                love.graphics.newQuad(248, 32, 8, 8,
                                      tilesetImage:getDimensions())
            },
            bottomRight = {
                love.graphics.newQuad(216, 40, 8, 8,
                                      tilesetImage:getDimensions()),
                love.graphics.newQuad(248, 40, 8, 8,
                                      tilesetImage:getDimensions())
            }
        },
        grass = {
            middle = love.graphics.newQuad(294, 86, 8, 8,
                                           tilesetImage:getDimensions())
        },
        longGrass = {
            topLeft = love.graphics.newQuad(16, 24, 8, 8,
                                            tilesetImage:getDimensions()),
            left = love.graphics.newQuad(16, 32, 8, 8,
                                         tilesetImage:getDimensions()),
            bottomLeft = love.graphics.newQuad(16, 40, 8, 8,
                                               tilesetImage:getDimensions()),
            top = love.graphics.newQuad(24, 24, 8, 8,
                                        tilesetImage:getDimensions()),
            middle = love.graphics.newQuad(24, 32, 8, 8,
                                           tilesetImage:getDimensions()),
            bottom = love.graphics.newQuad(24, 40, 8, 8,
                                           tilesetImage:getDimensions()),
            topRight = love.graphics.newQuad(32, 24, 8, 8,
                                             tilesetImage:getDimensions()),
            right = love.graphics.newQuad(32, 32, 8, 8,
                                          tilesetImage:getDimensions()),
            bottomRight = love.graphics.newQuad(32, 40, 8, 8,
                                                tilesetImage:getDimensions())
        },
        mud = {
            topLeft = love.graphics.newQuad(56, 24, 8, 8,
                                            tilesetImage:getDimensions()),
            left = love.graphics.newQuad(56, 32, 8, 8,
                                         tilesetImage:getDimensions()),
            bottomLeft = love.graphics.newQuad(56, 40, 8, 8,
                                               tilesetImage:getDimensions()),
            top = love.graphics.newQuad(64, 24, 8, 8,
                                        tilesetImage:getDimensions()),
            middle = love.graphics.newQuad(64, 32, 8, 8,
                                           tilesetImage:getDimensions()),
            bottom = love.graphics.newQuad(64, 40, 8, 8,
                                           tilesetImage:getDimensions()),
            topRight = love.graphics.newQuad(72, 24, 8, 8,
                                             tilesetImage:getDimensions()),
            right = love.graphics.newQuad(72, 32, 8, 8,
                                          tilesetImage:getDimensions()),
            bottomRight = love.graphics.newQuad(72, 40, 8, 8,
                                                tilesetImage:getDimensions())
        },
        stone = {
            topLeft = love.graphics.newQuad(128, 72, 8, 8,
                                            tilesetImage:getDimensions()),
            left = love.graphics.newQuad(128, 80, 8, 8,
                                         tilesetImage:getDimensions()),
            bottomLeft = love.graphics.newQuad(128, 88, 8, 8,
                                               tilesetImage:getDimensions()),
            top = love.graphics.newQuad(136, 72, 8, 8,
                                        tilesetImage:getDimensions()),
            middle = love.graphics.newQuad(136, 80, 8, 8,
                                           tilesetImage:getDimensions()),
            bottom = love.graphics.newQuad(136, 88, 8, 8,
                                           tilesetImage:getDimensions()),
            topRight = love.graphics.newQuad(144, 72, 8, 8,
                                             tilesetImage:getDimensions()),
            right = love.graphics.newQuad(144, 80, 8, 8,
                                          tilesetImage:getDimensions()),
            bottomRight = love.graphics.newQuad(144, 88, 8, 8,
                                                tilesetImage:getDimensions())
        }
    }
    propsImage = love.graphics.newImage("img/World/Props.png")
    propsQuads = {
        tree = love.graphics.newQuad(155, 3, 21, 25, propsImage:getDimensions()),
        plant1 = love.graphics.newQuad(74, 52, 4, 5, propsImage:getDimensions()),
        plant2 = love.graphics.newQuad(80, 51, 8, 6, propsImage:getDimensions()),
        plant3 = love.graphics.newQuad(89, 52, 4, 4, propsImage:getDimensions())
    }

end

function getMapTile(x, y)
    if not map[x] then return nil end
    return map[x][y]
end

function setMapTile(x, y, value)
    if not map[x] then map[x] = {} end
    map[x][y] = value
end

function generateTerrainType(x, y, seed)
    local noiseValue = love.math.noise(x * scale, y * scale)
    if noiseValue < 0.15 then
        return "water"
    elseif noiseValue < 0.3 then
        return "grass"
    elseif noiseValue < 0.7 then
        return "longGrass"
    elseif noiseValue < 0.85 then
        return "mud"
    else
        return "stone"
    end
end

function createNoiseMap(width, height)
    local noiseMap = {}
    for x = 1, width do
        noiseMap[x] = {}
        for y = 1, height do noiseMap[x][y] = math.random() end
    end
    return noiseMap
end

function getVisibleArea(playerX, playerY)
    local startX = math.floor((playerX - viewportWidth / 2) / tileSize) - buffer
    local startY = math.floor((playerY - viewportHeight / 2) / tileSize) -
                       buffer
    local endX = math.floor((playerX + viewportWidth / 2) / tileSize) + buffer
    local endY = math.floor((playerY + viewportHeight / 2) / tileSize) + buffer

    return startX, startY, endX, endY
end

function world.update(dt)
    entityManager.update(dt)
    player:update(dt)
    local startX, startY, endX, endY = getVisibleArea(player.x, player.y)

    waterFrameTimer = waterFrameTimer + dt
    if waterFrameTimer >= waterFrameDuration then
        currentWaterFrame = 3 - currentWaterFrame
        waterFrameTimer = waterFrameTimer - waterFrameDuration
    end

    for x = startX, endX do
        for y = startY, endY do
            if not getMapTile(x, y) then
                setMapTile(x, y, generateTerrainType(x, y, seed))
            end
        end
    end
    camera:update(dt, player.x, player.y)
    drawManager.update(dt)
end

function getTileType(x, y, neighbourType)
    local top = map[x] and map[x][y - 1] == neighbourType
    local bottom = map[x] and map[x][y + 1] == neighbourType
    local left = map[x - 1] and map[x - 1][y] == neighbourType
    local right = map[x + 1] and map[x + 1][y] == neighbourType

    if top and left then
        return "topLeft"
    elseif left and bottom then
        return "bottomLeft"
    elseif right and top then
        return "topRight"
    elseif right and bottom then
        return "bottomRight"
    elseif left then
        return "left"
    elseif right then
        return "right"
    elseif top then
        return "top"
    elseif bottom then
        return "bottom"
    else
        return "middle"
    end
end

function placeProp(x, y, terrainType)
    if terrainType == "longGrass" then
        local treeNoise = love.math.simplexNoise(x * 0.5, y * 0.5, propsSeed)
        if treeNoise > 0.50000005 and treeNoise < 0.50025 then
            local treeQuad = propsQuads["tree"]
            local quadWidth, quadHeight = 21, 25
            local scaledWidth, scaledHeight = quadWidth * tileScale,
                                              quadHeight * tileScale
            local offsetX, offsetY = scaledWidth / 2, scaledHeight / 2

            local drawX = (x - 1) * tileSize - offsetX
            local drawY = (y - 1) * tileSize - offsetY
            drawManager.bufferDrawEntity(drawX, drawY + (2 * offsetY),
                                         function()
                love.graphics.draw(propsImage, treeQuad, drawX, drawY, 0,
                                   tileScale, tileScale)
            end)

        end
    elseif terrainType == "grass" then
        local plantNoise = love.math.simplexNoise(x, y, propsSeed)
        local plantQuad = nil
        local quadWidth, quadHeight = 0, 0
        if plantNoise > 0.1 and plantNoise < 0.15 then
            plantQuad = propsQuads["plant1"]
            quadWidth, quadHeight = 4, 5
        elseif plantNoise > 0.6 and plantNoise < 0.65 then
            plantQuad = propsQuads["plant2"]
            quadWidth, quadHeight = 8, 6
        elseif plantNoise < 0.05 then
            plantQuad = propsQuads["plant3"]
            quadWidth, quadHeight = 4, 4
        end
        if plantQuad then
            local scaledWidth, scaledHeight = quadWidth * tileScale,
                                              quadHeight * tileScale
            local offsetX, offsetY = scaledWidth / 2, scaledHeight / 2

            local drawX = (x - 1) * tileSize - offsetX
            local drawY = (y - 1) * tileSize - offsetY
            drawManager.bufferDrawEntity(drawX, drawY + (2 * offsetY),
                                         function()
                love.graphics.draw(propsImage, plantQuad, drawX, drawY, 0,
                                   tileScale, tileScale)
            end)
        end
    end
    return false
end

function world.draw()
    camera:apply()
    local startX, startY, endX, endY = getVisibleArea(player.x, player.y)
    for x = startX, endX do
        for y = startY, endY do
            local terrainType = getMapTile(x, y)
            if terrainType then
                if terrainType == "water" then
                    local waterType = getTileType(x, y, "grass")
                    love.graphics.draw(tilesetImage,
                                       tileQuads["water"][waterType][currentWaterFrame],
                                       (x - 1) * tileSize, (y - 1) * tileSize,
                                       0, tileScale, tileScale)
                elseif terrainType == "grass" then
                    love.graphics.draw(tilesetImage,
                                       tileQuads["grass"]["middle"],
                                       (x - 1) * tileSize, (y - 1) * tileSize,
                                       0, tileScale, tileScale)
                elseif terrainType == "longGrass" then
                    local longGrassType = getTileType(x, y, "grass")
                    love.graphics.draw(tilesetImage,
                                       tileQuads["longGrass"][longGrassType],
                                       (x - 1) * tileSize, (y - 1) * tileSize,
                                       0, tileScale, tileScale)
                elseif terrainType == "mud" then
                    local mudType = getTileType(x, y, "longGrass")
                    love.graphics.draw(tilesetImage, tileQuads["mud"][mudType],
                                       (x - 1) * tileSize, (y - 1) * tileSize,
                                       0, tileScale, tileScale)
                elseif terrainType == "stone" then
                    local stoneType = getTileType(x, y, "mud")
                    love.graphics.draw(tilesetImage,
                                       tileQuads["stone"][stoneType],
                                       (x - 1) * tileSize, (y - 1) * tileSize,
                                       0, tileScale, tileScale)
                end
            end
        end
    end
    for y = startY, endY do
        for x = startX, endX do
            local terrainType = getMapTile(x, y)
            placeProp(x, y, terrainType)
        end
    end
    entityManager.draw()
    player:draw()
    drawManager.draw()
    love.graphics.origin()
end

return world
